/****************************************************************************/
/*  SETUP_CONFIG module library                                             */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/****************************************************************************/
/*  Library - SETUP_CONFIG.cpp file                                         */
/****************************************************************************/
/*  Tested on Nucleo-L476RG / 11th nov 2021                                 */
/****************************************************************************/
/*  SetUp for CeTI Introduction with KORG Electribe Sampler                 */
/*  Light Type                                                              */
/*      SLS-6 - 7 ch - DIM / STR / R / G / B / PRES / PROG                  */
/*      LVPT12 - 7 ch - MODE(0) / COLOR / SPEED / DIM / R / G / B           */
/*  Light SetUp :                                                           */
/*      instru  8   ()    SLS-6         DMX : 128                           */
/*      instru  9   ()    SLS-6         DMX : 136                           */
/*      instru  10  ()    SLS-6         DMX : 144                           */
/*      instru  12  ()    LVPT12        DMX : 0                             */
/*      instru  13  ()    LVPT12        DMX : 8                             */
/*      instru  14  ()    LVPT12        DMX : 16                            */
/*      instru  15  ()    LVPT12        DMX : 24                            */

#include    "SETUP_CONFIG.h"
#include    "DMX_MIDI.h"

/* Fonctions */  
/* Detection d'une note reçue en MIDI */
void detectNoteMIDI_setup(void){
    uint8_t     temp_cpt;
    short       adr;

    if(new_note_midi == 1){         // NOTE_ON
        if(DEBUG_MODE)  printf("%x - C=%x - %d %d \r\n", midi_data[0], channels_int_on, note_data, velocity_data);
        for(uint8_t channel = 0; channel < 16; channel++){
            if((channels_int_on & ((0b1) << channel)) != 0){
                for(temp_cpt = 0; temp_cpt < channel_ad_nb[channel][1]; temp_cpt++){
                    adr = channel_ad_nb[channel][0];
                    if(temp_cpt < 8){
                        if(temp_cpt != channel_dim[channel][0] - 1){      // All channels except Dimmer channel
                            dmx_data[adr + temp_cpt] = channel_dmxConfig[channel][temp_cpt];
                        }
                    }    
                }
            }
        }
        channels_int_on = 0;
    }
    if(new_note_midi == 2){        // NOTE_OFF
        if(DEBUG_MODE)  printf("N = %x - C=%d - %d %d \r\n", midi_data[0], channel_data, note_data, velocity_data);
        for(uint8_t channel = 0; channel < 16; channel++){
            if((channels_int_off & ((0b1) << channel)) != 0){
                for(temp_cpt = 0; temp_cpt < channel_ad_nb[channel][1]; temp_cpt++){
                    adr = channel_ad_nb[channel][0];
                    if(temp_cpt != channel_dim[channel][0] - 1){      // All channels except Dimmer channel
                        if(temp_cpt < 8)    dmx_data[adr + temp_cpt] = 0;
                    }
                }
            }
        }
        channels_int_off = 0;    
    }
    if(new_note_midi != 0){
        new_note_midi = 0;  
    }
}

/* Detection d'une donnée reçue en MIDI */
void detectDataMIDI_setup(void){
    if(new_data_midi == 1){         // NOTE_ON
        if(DEBUG_MODE)  printf("D = %x - C=%d - %d %d \r\n", midi_data[0], channel_data, note_data, velocity_data);
    }
        
    if(new_data_midi != 0){
        new_data_midi = 0;  
    }
}

/* Detection d'un controle continu reçu en MIDI */
void detectContCtlMIDI_setup(void){
    if(new_CC_midi == 1){
        switch(midi_data[1]){
            case    0x07    :  // Main Volume
                /*
                for(uint8_t channel = 0; channel < 16; channel++){
                    dmx_data[channel_ad_nb[channel][0] + channel_dim[channel] - 1] = midi_data[2];
                }
                */
                break;
            case    0x49   :  // Attack Time Value
                for(uint8_t channel = 0; channel < 16; channel++){
                    uint16_t adr = channel_ad_nb[channel][0];
                    uint8_t dim_ch = channel_dim[channel][0];
                    uint8_t relative_intensity = channel_dim[channel][1];
                    if(dim_ch != 0){
                        uint16_t ch = adr + dim_ch - 1;
                        dmx_data[ch] = (2 * midi_data[2]) * relative_intensity / 100;
                    }                    
                }
                break;
            default :
                debug_out = debug_out;
        }
        new_CC_midi = 0;
    }
}