/****************************************************************************/
/*  SETUP_CONFIG module library                                             */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/****************************************************************************/
/*  Library - SETUP_CONFIG.h file                                           */
/****************************************************************************/
/*  Tested on Nucleo-L476RG / 11th nov 2021                                 */
/****************************************************************************/
/*  SetUp for CeTI Introduction with KORG Electribe Sampler                 */
/*  Light Type                                                              */
/*      SLS-6 - 7 ch - DIM / STR / R / G / B / PRES / PROG                  */
/*      LVPT12 - 7 ch - MODE(0) / COLOR / SPEED / DIM / R / G / B           */
/*      STRB_Euro - 7 ch - DIM / R / G / B / W / STR / MODE(0)              */
/*  Light SetUp :                                                           */
/*      instru  8   ()    SLS-6         DMX : 128                           */
/*      instru  9   ()    SLS-6         DMX : 136                           */
/*      instru  10  ()    SLS-6         DMX : 144                           */
/*      instru  11  ()    SLS-6         DMX : 152                           */
/*      instru  12  ()    STRB_Euro     DMX : 0                             */
/*      instru  13  ()    STRB_Euro     DMX : 8                             */
/*      instru  14  ()    STRB_Euro     DMX : 16                            */
/*      instru  15  ()    STRB_Euro     DMX : 24                            */


#ifndef     SETUP_CONFIG_H_INCLUDED
#define     SETUP_CONFIG_H_INCLUDED
#include    "mbed.h"

#define     DEBUG_MODE      false


/* Variables globales */
const short             channel_ad_nb[16][2] = {       // spot adress and dmx chan number for instrument at MIDI channel
        {   0,  0},     {   0,  0},     {   0,  0},     {   0,  0},
        {   0,  0},     {   0,  0},     {   0,  0},     {   0,  0},
        {   128,  7},   {   136,  7},   {   144,  7},   {   152,  7},
        {   16,  7},     {   24,  7},     {   8,  7},    {   0,  7}};

const uint8_t           channel_dim[16][2] = {       // DMX dimmer channel number for instrument at MIDI channel and relative intensity (0 - 100%)
        {   0,  0},     {   0,  0},     {   0,  0},     {   0,  0},
        {   0,  0},     {   0,  0},     {   0,  0},     {   0,  0},
        {   1,  100},   {   1,  70},    {   1,  80},    {   1,  80},
        {   1,  80},    {   1,  80},    {   1,  80},    {   1,  80}};

const uint8_t           channel_dmxConfig[16][8] = {    // DMX value for each DMX channel (first byte) for instrument at MIDI channel
        {   0,  0,  0,  0,  0,  0,  0,  0},     // MIDI channel 0
        {   0,  0,  0,  0,  0,  0,  0,  0},
        {   0,  0,  0,  0,  0,  0,  0,  0},
        {   0,  0,  0,  0,  0,  0,  0,  0},
        {   0,  0,  0,  0,  0,  0,  0,  0},     // MIDI channel 4
        {   0,  0,  0,  0,  0,  0,  0,  0},
        {   0,  0,  0,  0,  0,  0,  0,  0},
        {   0,  0,  0,  0,  0,  0,  0,  0},     
        {   255,  0,  255,  0,  0,  0,  0,  0},     // MIDI channel 8   - DIM / STR / R / G / B / PRES / PROG 
        {   255,  0,  128,  64,  196,  0,  0,  0},
        {   255,  0,  64,  255,  64,  0,  0,  0},
        {   255,  0,  128,  128,  64,  0,  0,  0},
        {   255,  255,  128, 0,  0,  0,  0,  0},     // MIDI channel 12  - MODE(0) / COLOR / SPEED / DIM / R / G / B
        {   255,  128,  196, 64,  0,  0,  0,  0},
        {   255,  64,  128, 196,  0,  0,  0,  0},
        {   255,  128,  64,  196,  0,  0,  0,  0}};


/* Fonctions */  
/* Detection d'une note reçue en MIDI */
void detectNoteMIDI_setup(void);
/* Detection d'une donnée reçue en MIDI */
void detectDataMIDI_setup(void);
/* Detection d'un controle continu reçu en MIDI */
void detectContCtlMIDI_setup(void);

#endif